////////////////////////////////////////////////////////////////////////////////////////

string bullet_mdl = <rocket.mdl>; // always define ent_created entities as strings!
string muzzle_pcx = <muzzle.pcx>; // muzzle sprite
string explosion_pcx = <explosion+5.pcx>; // explosion sprite

////////////////////////////////////////////////////////////////////////////////////////

sound bullet_wav = <bullet.wav>; // bullet sound

////////////////////////////////////////////////////////////////////////////////////////

function fire_bullets(); // creates the bullets
function move_bullets(); // moves the bullets
function remove_bullets(); // removes the bullets after they've hit something
function display_muzzle(); // shows the gun sprite
function explosion_sprite(); // controls the explosion sprite

////////////////////////////////////////////////////////////////////////////////////////

entity* weapon1; // pointer to our weapon model

////////////////////////////////////////////////////////////////////////////////////////

action players_weapon
{
	weapon1 = my; // I'm weapon1
	var player1_pos; // stores the initial position of the player
	var player2_pos; // stores the position of the player after a frame
	var weapon_height;
	var weapon_offset;
	my.passable = on; // the weapon model is passable
	while (player == null) {wait (1);} // wait until the player is created
	while (1)
	{
		vec_set (player1_pos.x, player.x); // store the initial player position
		// place the weapon 50 quants in front, 18 quants to the right and 20 quants below camera's origin
		vec_set (weapon_offset.x, vector (50, -18, -20));
		if (vec_dist (player1_pos.x, player2_pos.x) != 0) // the player has moved during the last frame?
		{
			weapon_height += 30 * time; // then offset weapon_height (30 = weapon waving speed)
			weapon_offset.z += 0.3 * sin(weapon_height); // (0.3 = weapon waving amplitude)
		}
		// rotate weapon_offset according to the camera angles
		vec_rotate (weapon_offset.x, vector (camera.pan, camera.tilt, 0)); 
		vec_add (weapon_offset.x, camera.x); // add the camera position to weapon_offset
		vec_set (my.x, weapon_offset.x); // set weapon's coords to weapon_offset
		my.pan = camera.pan; // use the same camera angles for the weapon
		my.tilt = camera.tilt;
		vec_set (player2_pos.x, player.x); // store the new player coordinates after 1 frame
		wait (1);
	}
}

on_mouse_left = fire_bullets; // call function fire_bullets() when the left mouse button is pressed

function fire_bullets()
{
	proc_kill(4); // don't allow more than 1 copy of this function to run
	while (mouse_left == on) // this loop runs for as long as the left mouse button is pressed
	{
//		vp-=0.01;
		vec_for_vertex (temp.x, weapon1, 29); // get the coordinates for the bullets' origin
		 // create the bullet at camera's position and attach it the "move_bullets" function
		ent_create (bullet_mdl, temp.x, move_bullets);
		ent_create (muzzle_pcx, temp.x, display_muzzle); // create the gun muzzle
		snd_play (bullet_wav, 100, 0); // play the bullet sound at a volume of 100
		sleep (0.14); // fire 7 bullets per second (0.14 * 7 = 1 second)
	}
//	vp=6;
}

function move_bullets()
{
	var bullet_speed; // this var will store the speed of the bullet
	
	my.skill3=1;
	my.enable_impact = on; // the bullet is sensitive to impact
	my.enable_entity = on; // and to other entities
	my.enable_block = on; // as well as to level blocks
	my.event = remove_bullets; // when it collides with something, its event function (remove_bullets) will run
	my.pan = camera.pan; // the bullet has the same pan
	my.tilt = camera.tilt; // and tilt with the camera
	bullet_speed.x = 200 * time; // adjust the speed of the bullet here
	bullet_speed.y = 0; // the bullet doesn't move sideways
	bullet_speed.z = 0; // then don't allow the gravity to have its ways with the bullet
	while (my != null) // this loop will run for as long as the bullet exists (it isn't "null")
	{
		// move the bullet ignoring the passable entities and store the result in distance_covered
		c_move (my, bullet_speed, nullvector, ignore_passable);
		wait (1);
	}
}

function remove_bullets() // this function runs when the bullet collides with something
{
	wait (1); // wait a frame to be sure (don't trigger engine warnings)
	ent_create (explosion_pcx, my.x, explosion_sprite); // create the explosion sprite
	my.passable = on; // the bullets becomes passable and invisible now
	my.invisible = on; // so it can't harm anyone anymore
	sleep (2); // wait until the explosion_sprite() function is over
	ent_remove (my); // and then remove the bullet
}

function display_muzzle()
{
	my.passable = on; // the muzzle is passable
	my.flare = on; // make the black parts transparent even without using tga files (using a pcx file here)
	my.bright = on; // the muzzle should be bright
	my.ambient = 100; // and this line makes it even brighter
	my.oriented = on; // the sprite is oriented
	my.pan = weapon1.pan; // has the same pan and tilt
	my.tilt = weapon1.tilt; // with the weapon
	my.roll = random(360); // and a random roll angle (looks nicer)
	my.scale_x = 0.5; // we scale it down
	my.scale_y = my.scale_x; // on all the axis
	my.scale_z = my.scale_z; // this would only be needed for a 3D (model-based) muzzle
	weapon1.ambient = 100; // highlight the weapon (makes it look more realistic)
	wait (2); // show the muzzle sprite for 2 frames
	weapon1.ambient = 0; // restore the normal weapon ambient value
	ent_remove (my); // and then remove it
}

function explosion_sprite()
{
	my.passable = on; // the explosion sprite is passable
	my.scale_x = 0.3; // we scale it down to 0.15
	my.scale_y = my.scale_x; // on both axis
	my.flare = on; // we set its flare
	my.bright = on; // and bright flags
	my.ambient = 100; // and we give it an ambient of 100
	my.roll = random(360); // we set a random roll angle
	my.transparent = on; // and make it transparent
	my.alpha = 100; // but we set it to be completely opaque for now
	while (my.frame < 5) // go through all the animation frames
	{
		my.frame += 2 * time; // animation speed
      	wait (1);
	}
	while (my.alpha > 0) // now fade the last frame quickly
	{
		my.alpha -= 50 * time; // 50 = fading speed
		wait (1);
	}
	ent_remove (my);
}